@@ -19,6 +19,10 @@
 #include "bookmarksdialog.h"
 #include "settings.h"
 
+#ifdef DORIAN_TEST_MODEL
+#include "modeltest.h"
+#endif
+
 #ifdef Q_WS_MAC
 #   define ICON_PREFIX ":/icons/mac/"
 #else
@@ -30,7 +34,7 @@ const Qt::WindowFlags WIN_BIG_FLAGS =
 const int WIN_BIG_TIMER = 3000;
 
 MainWindow::MainWindow(QWidget *parent):
-        QMainWindow(parent), view(0), book(0), isFullscreen(false)
+        QMainWindow(parent), view(0), isFullscreen(false)
 {
 #ifdef Q_WS_MAEMO_5
     setAttribute(Qt::WA_Maemo5StackedWindow, true);
@@ -76,7 +80,7 @@ MainWindow::MainWindow(QWidget *parent):
                                         "view-fullscreen");
 
     // Handle model changes
-    connect(Library::instance(), SIGNAL(currentBookChanged()),
+    connect(Library::instance(), SIGNAL(nowReadingChanged()),
             this, SLOT(onCurrentBookChanged()));
 
     normalFlags = windowFlags();
@@ -87,21 +91,21 @@ MainWindow::MainWindow(QWidget *parent):
     if (QCoreApplication::arguments().size() == 2) {
         QString path = QCoreApplication::arguments()[1];
         library->add(path);
-        int index = library->find(path);
-        if (index != -1) {
-            library->setCurrent(index);
+        QModelIndex index = library->find(path);
+        if (index.isValid()) {
+            library->setNowReading(index);
         }
     }
     else {
-        Book *current = library->current();
-        if (current) {
-            setCurrentBook(current);
+        QModelIndex index = library->nowReading();
+        if (index.isValid()) {
+            library->setNowReading(index);
         }
         else {
-            if (!library->size()) {
+            if (!library->rowCount()) {
                 library->add(":/books/2 B R 0 2 B.epub");
             }
-            library->setCurrent(0);
+            library->setNowReading(library->index(0));
         }
     }
 
@@ -112,12 +116,19 @@ MainWindow::MainWindow(QWidget *parent):
                                    Settings::instance()->value("orientation"));
 
     // Handle loading chapters
-    connect(view, SIGNAL(chapterLoaded(int)), this, SLOT(onChapterLoaded(int)));
+    connect(view, SIGNAL(chapterLoadStart(int)),
+            this, SLOT(onChapterLoadStart()));
+    connect(view, SIGNAL(chapterLoadEnd(int)),
+            this, SLOT(onChapterLoadEnd(int)));
+
+#ifdef DORIAN_TEST_MODEL
+    (void)new ModelTest(Library::instance(), this);
+#endif
 }
 
 void MainWindow::onCurrentBookChanged()
 {
-    setCurrentBook(Library::instance()->current());
+    setCurrentBook(Library::instance()->nowReading());
 }
 
 void MainWindow::showNormal()
@@ -143,14 +154,16 @@ void MainWindow::showFullScreen()
     restoreButton->flash();
 }
 
-void MainWindow::setCurrentBook(Book *current)
+void MainWindow::setCurrentBook(const QModelIndex &current)
 {
-    book = current;
-    view->setBook(current);
-    setWindowTitle(current? current->title: "Dorian");
+    mCurrent = current;
+    Book *book = Library::instance()->book(current);
+    view->setBook(book);
+    setWindowTitle(book? book->name(): tr("Dorian"));
 }
 
-QAction *MainWindow::addToolBarAction(const QObject *receiver, const char *member,
+QAction *MainWindow::addToolBarAction(const QObject *receiver,
+                                      const char *member,
                                       const QString &name)
 {
     return toolBar->
@@ -159,8 +172,8 @@ QAction *MainWindow::addToolBarAction(const QObject *receiver, const char *membe
 
 void MainWindow::showLibrary()
 {
-    LibraryDialog *dialog = new LibraryDialog();
-    dialog->exec();
+    LibraryDialog *dialog = new LibraryDialog(this);
+    dialog->show();
 }
 
 void MainWindow::showSettings()
@@ -171,8 +184,9 @@ void MainWindow::showSettings()
 
 void MainWindow::showInfo()
 {
-    if (book) {
-        InfoDialog *info = new InfoDialog(book, this);
+    if (mCurrent.isValid()) {
+        InfoDialog *info =
+            new InfoDialog(Library::instance()->book(mCurrent), this);
         info->exec();
     }
 }
@@ -185,6 +199,7 @@ void MainWindow::showDevTools()
 
 void MainWindow::showBookmarks()
 {
+    Book *book = Library::instance()->book(mCurrent);
     if (book) {
         BookmarksDialog *bookmarks = new BookmarksDialog(book, this);
         int ret = bookmarks->exec();
@@ -246,10 +261,18 @@ void MainWindow::onSettingsChanged(const QString &key)
 #endif // Q_WS_MAEMO_5
 }
 
-void MainWindow::onChapterLoaded(int index)
+void MainWindow::onChapterLoadStart()
+{
+#ifdef Q_WS_MAEMO_5
+    setAttribute(Qt::WA_Maemo5ShowProgressIndicator, true);
+#endif
+}
+
+void MainWindow::onChapterLoadEnd(int index)
 {
     bool enablePrevious = false;
     bool enableNext = false;
+    Book *book = Library::instance()->book(mCurrent);
     if (book) {
         if (index > 0) {
             enablePrevious = true;
@@ -259,6 +282,7 @@ void MainWindow::onChapterLoaded(int index)
         }
     }
 #ifdef Q_WS_MAEMO_5
+    setAttribute(Qt::WA_Maemo5ShowProgressIndicator, false);
     previousAction->setIcon(QIcon(enablePrevious?
         ":/icons/previous.png" : ":/icons/previous-disabled.png"));
     nextAction->setIcon(QIcon(enableNext?
