@@ -4,6 +4,7 @@
 #include <QtGui>
 
 class QString;
+class QModelIndex;
 class DevTools;
 class BookView;
 class Book;
@@ -27,7 +28,8 @@ public slots:
     void showNormal();
     void showFullScreen();
     void onSettingsChanged(const QString &key);
-    void onChapterLoaded(int index);
+    void onChapterLoadStart();
+    void onChapterLoadEnd(int index);
 
 protected:
 #ifdef Q_WS_MAEMO5
@@ -40,7 +42,7 @@ protected:
     virtual void closeEvent(QCloseEvent *event);
 
 private:
-    void setCurrentBook(Book *book);
+    void setCurrentBook(const QModelIndex &current);
     QAction *addToolBarAction(const QObject *receiver, const char *member,
                               const QString &name);
     QRect fullScreenZone() const;
@@ -58,7 +60,7 @@ private:
     QToolBar *toolBar;
     QDialog *settings;
     DevTools *devTools;
-    Book *book;
+    QModelIndex mCurrent;
     Qt::WindowFlags normalFlags;
     TranslucentButton *restoreButton;
     bool isFullscreen;
